#include <extdll.h>			// always
#include <meta_api.h>		// of course
#include <dllapi.h>
#include "sdk_util.h"		// UTIL_LogPrintf, etc
#include <h_export.h>
#include "plugin.h"
#include "cbase.h"
#include <usercmd.h>
#include <pm_defs.h>

enginefuncs_t g_engfuncs;
globalvars_t  *gpGlobals;

pdata_t	PlayerData[33];
bool BlockDeathMsg;
int CurrentMessage;

cvar_t nsa_manualsiege				= {"nsa_manualsiege",   "1",    FCVAR_SERVER}; 
cvar_t nsa_siege_damage				= {"nsa_siege_damage",   "300",  FCVAR_SERVER}; 
cvar_t nsa_siege_range				= {"nsa_siege_range",    "2048", FCVAR_SERVER}; 
cvar_t nsa_siege_delay				= {"nsa_siege_delay",	"5",    FCVAR_SERVER}; 
cvar_t nsa_siege_radius				= {"nsa_siege_radius",   "300",  FCVAR_SERVER}; 


void CmdStart( const edict_t *player, const struct usercmd_s *_cmd, unsigned int random_seed )
{
	edict_t * pEntity = (edict_t *)player;
	
	struct usercmd_s * cmd     = (struct usercmd_s *)_cmd;

	if(PlayerData[ENTINDEX(pEntity)].m_WeaponHolstered)
	{
		PlayerData[ENTINDEX(pEntity)].m_Attack = (cmd->buttons & IN_ATTACK); //So we can tell if they are attacking.
		cmd->buttons &= ~IN_ATTACK;											 //block thier attack.
		RETURN_META(MRES_SUPERCEDE);
	}
	RETURN_META(MRES_IGNORED);
}

BOOL ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] )
{
	PlayerData[ENTINDEX(pEntity)].m_Attack = 0;
	PlayerData[ENTINDEX(pEntity)].m_SiegeUsing = 0;
	PlayerData[ENTINDEX(pEntity)].m_WeaponHolstered = 0;
	RETURN_META_VALUE(MRES_HANDLED,0);

}
void PlayerPreThink(edict_t *pEntity)
{
	if(PlayerData[ENTINDEX(pEntity)].m_SiegeUsing)
	{
		edict_t *pSiege = INDEXENT(	PlayerData[ENTINDEX(pEntity)].m_SiegeUsing );
		if(FNullEnt(pSiege) || !UTIL_HasAdvTF(pSiege))//Is the siege gone? or is there no AdvTF in range?
		{
			UTIL_DeployWeapon(pEntity);
			
			PlayerData[ENTINDEX(pEntity)].m_SiegeUsing = 0;
			
			CBaseEntity *pSpot = NULL;
			while (pSpot = UTIL_FindEntityByClassname( pSpot, "siege_spot" ))
			{
				if(pSpot->pev->euser1 == pEntity)
					pSpot->pev->flags = FL_KILLME;
			}
			
			if(pSiege != NULL)//Hmm.. forgot this and it was a crash-o-thon.
				pSiege->v.euser1 = NULL;
		}
		RETURN_META(MRES_HANDLED);
	}

	if(PlayerData[ENTINDEX(pEntity)].m_WeaponHolstered)//Just to be safe. (Sorry, all you exploiters cant have a weapon model when you use the siege)
	{
		if(pEntity->v.weaponmodel || pEntity->v.viewmodel)
		{
			UTIL_HolsterWeapon(pEntity);
		}
		RETURN_META(MRES_HANDLED);
	}
	RETURN_META(MRES_IGNORED);
}

int DispatchSpawn(edict_t *pEntity)
{
	if(FStrEq(STRING(pEntity->v.classname), "worldspawn"))
	{
		PRECACHE_MODEL("sprites/glow02.spr");
		RETURN_META_VALUE(MRES_HANDLED,0);
	}
	RETURN_META_VALUE(MRES_IGNORED,0);
}

void DispatchThink(edict_t *pent)
{
	if(CVAR_GET_FLOAT("nsa_manualsiege"))
	{
		if(FStrEq(STRING(pent->v.classname), "siegeturret"))
		{
			if(UTIL_IsBuilt(pent) && !pent->v.iuser2)//iuser2 is 1 if we already set its new USE on it. We dont want to change the USE function if it hasnt been built!
			{
				CBaseEntity *pEnt = CBaseEntity::Instance(pent);
				pEnt->m_pfnUse =  &CBaseEntity::m_pfnSiegeUse;
				pent->v.iuser2 = 1;
			}
			pent->v.nextthink = 0.1;
		}
	}
	RETURN_META(MRES_HANDLED);
}

void ClientCommand(edict_t *pEntity)
{
	//Do not allow them to pick out a weapon, even if they can they still cant shoot it. but this makes it stay nice and neat :P
	if(PlayerData[ENTINDEX(pEntity)].m_WeaponHolstered)
	{
		if(!strncmp(CMD_ARGV(1), "weapon_", 7) || !strncmp(CMD_ARGV(0), "weapon_", 7) || FStrEq(CMD_ARGV(0), "lastinv"))
		{
			RETURN_META(MRES_SUPERCEDE);
		}
	}
	RETURN_META(MRES_IGNORED);
}

void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed)
{
	CurrentMessage = msg_type;

	if(CurrentMessage == gmsgDeathMsg && BlockDeathMsg)
		RETURN_META(MRES_SUPERCEDE);
	RETURN_META(MRES_IGNORED);
}

void MessageEnd( void )
{
	if(CurrentMessage == gmsgDeathMsg && BlockDeathMsg)
	{
		CurrentMessage = 0;
		RETURN_META(MRES_SUPERCEDE);
	}
	CurrentMessage = 0;
	RETURN_META(MRES_IGNORED);
}

void WriteByte(int iValue)
{
	if(CurrentMessage == gmsgDeathMsg && BlockDeathMsg)
		RETURN_META(MRES_SUPERCEDE);
	RETURN_META(MRES_IGNORED);
}

void WriteString(const char *sz)
{
	if(CurrentMessage == gmsgDeathMsg && BlockDeathMsg)
		RETURN_META(MRES_SUPERCEDE);
	RETURN_META(MRES_IGNORED);
}

void AlertMessage(ALERT_TYPE atype, char *szFmt, ...)
{
	if(atype == at_logged && BlockDeathMsg)
		RETURN_META(MRES_SUPERCEDE);
	RETURN_META(MRES_IGNORED);
}

// Must provide at least one of these..
static META_FUNCTIONS gMetaFunctionTable = {
		NULL,			// pfnGetEntityAPI				HL SDK; called before game DLL
		NULL,			// pfnGetEntityAPI_Post			META; called after game DLL
		GetEntityAPI2,	// pfnGetEntityAPI2				HL SDK2; called before game DLL
		NULL,			// pfnGetEntityAPI2_Post		META; called after game DLL
		NULL,			// pfnGetNewDLLFunctions		HL SDK2; called before game DLL
		NULL,			// pfnGetNewDLLFunctions_Post	META; called after game DLL
		GetEngineFunctions,			// pfnGetEngineFunctions		META; called before HL engine
		NULL,			// pfnGetEngineFunctions_Post	META; called after HL engine
};

// Description of plugin
plugin_info_t Plugin_info = {
	META_INTERFACE_VERSION,	// ifvers
		"Manual Siege",	// name
		"1.3",	// version
		"4/24/2003",	// date
		"voogru <voogru@voogru.com>",	// author
		"http://www.voogru.com",	// url
		"MANUALSIEGE",	// logtag
		PT_STARTUP,// (when) loadable
		PT_STARTUP,// (when) unloadable
};

// Global vars from metamod:
meta_globals_t *gpMetaGlobals;		// metamod globals
gamedll_funcs_t *gpGamedllFuncs;	// gameDLL function tables
mutil_funcs_t *gpMetaUtilFuncs;		// metamod utility functions

C_DLLEXPORT int Meta_Query(char *ifvers, plugin_info_t **pPlugInfo,	mutil_funcs_t *pMetaUtilFuncs) {
	*pPlugInfo=&Plugin_info;
	gpMetaUtilFuncs=pMetaUtilFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Attach(PLUG_LOADTIME now, META_FUNCTIONS *pFunctionTable, meta_globals_t *pMGlobals, gamedll_funcs_t *pGamedllFuncs) {
	gpMetaGlobals=pMGlobals;
	memcpy(pFunctionTable, &gMetaFunctionTable, sizeof(META_FUNCTIONS));
	gpGamedllFuncs=pGamedllFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Detach(PLUG_LOADTIME now, PL_UNLOAD_REASON reason) 
{
	return(TRUE);
}

void WINAPI GiveFnptrsToDll( enginefuncs_t* pengfuncsFromEngine, globalvars_t *pGlobals ) {
	memcpy(&g_engfuncs, pengfuncsFromEngine, sizeof(enginefuncs_t));
	gpGlobals = pGlobals;
		
	CVAR_REGISTER(&nsa_manualsiege);
	CVAR_REGISTER(&nsa_siege_damage);
	CVAR_REGISTER(&nsa_siege_range);
	CVAR_REGISTER(&nsa_siege_delay);
	CVAR_REGISTER(&nsa_siege_radius);
}

DLL_FUNCTIONS gFunctionTable;
C_DLLEXPORT int GetEntityAPI2( DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion ) 
{
	
	gFunctionTable.pfnCmdStart = CmdStart;
	gFunctionTable.pfnClientConnect = ClientConnect;
	gFunctionTable.pfnPlayerPreThink = PlayerPreThink;
	gFunctionTable.pfnSpawn = DispatchSpawn;
	gFunctionTable.pfnThink = DispatchThink;
	gFunctionTable.pfnClientCommand = ClientCommand;

	memcpy( pFunctionTable, &gFunctionTable, sizeof( DLL_FUNCTIONS ) );
	return(TRUE);
}



enginefuncs_t meta_engfuncs;
C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion ) 
{
	meta_engfuncs.pfnMessageBegin = MessageBegin;
	meta_engfuncs.pfnMessageEnd = MessageEnd;
	meta_engfuncs.pfnWriteByte = WriteByte;
	meta_engfuncs.pfnWriteString = WriteString;
	meta_engfuncs.pfnAlertMessage = AlertMessage;

	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
	return(TRUE);
}
